#include "crutil.hh"
#include <istream>
#include <algorithm>
#include <string>

bool _CRUTIL::is_substr_i(const std::string& sup, const std::string& sub,
                          int pos) {
    if (pos < 0) {
        pos = pos + 1 + sub.size() - sub.size();
    }
    if (pos < 0 || pos + sub.size() > sup.size()) {
        return false;
    }
    const char *psup = sup.c_str() + pos, *psub = sub.c_str();
    while (*psub) {
        char c1 = *psup++, c2 = *psub++;
        if (c1 < 'a') {
            c1 += 'a' - 'A';
        }
        if (c2 < 'a') {
            c2 += 'a' - 'A';
        }
        if (c1 != c2) {
            return false;
        }
    }
    return true;
}

int _CRUTIL::strncasecmp(const char* s1, const char* s2, size_t n) {
    while (n--) {
        char c1 = *s1++, c2 = *s2++;
        int diff = c2 - c1;
        if (diff == 0) {
            if (!c1) {
                break;
            }
            continue;
        } else if (diff == 32 && c1 >= 'A' && c1 <= 'Z') {
            continue;
        } else if (diff == -32 && c2 >= 'A' && c2 <= 'Z') {
            continue;
        } else {
            return diff > 0 ? 1 : -1;
        }
    }
    return 0;
}

_CRUTIL::FileParser::FileParser(const std::string& filename)
    : std::ifstream(filename), _line(""), _filename(filename), _line_number(0) {
    if (!good()) {
        throw std::runtime_error("unable to read '" + filename + "'");
    }
}
_CRUTIL::FileParser& _CRUTIL::FileParser::next_line(skip s) {
    while (std::getline(*this, _line)) {
        ++_line_number;
        // trim line
        _line.erase(_line.begin(), std::find_if(_line.begin(), _line.end(),
                                                [](unsigned char ch) {
                                                    return !std::isspace(ch);
                                                }));
        _line.erase(
            std::find_if(_line.rbegin(), _line.rend(),
                         [](unsigned char ch) { return !std::isspace(ch); })
                .base(),
            _line.end());

        if (_line.empty() ? !(s & skip::empty)
                          : _line[0] != '#' || !(s & skip::hashtag)) {
            break;
        }
    }
    return *this;
}
